import {Module} from 'vuex'
import {RootState} from '@/store/store'

interface UiState {
  sideBarCollapsed: boolean
  darkMode: boolean
  darkModeUnlocked: boolean
  showJumbotron: boolean
}

function initialState (): UiState {
  return {
    sideBarCollapsed: false,
    darkMode: false,
    darkModeUnlocked: false,
    showJumbotron: true
  }
}

export const uiMut = Object.freeze({
  SET_SIDEBAR_COLLAPSED: 'SET_SIDEBAR_COLLAPSED',
  SET_DARK_MODE: 'SET_DARK_MODE',
  SET_DARK_MODE_UNLOCKED: 'SET_DARK_MODE_UNLOCKED',
  SET_SHOW_JUMBOTRON: 'SET_SHOW_JUMBOTRON'
})

const ui: Module<UiState, RootState> = {
  state: initialState(),
  mutations: {
    [uiMut.SET_SIDEBAR_COLLAPSED] (state, collapsed: boolean) {
      state.sideBarCollapsed = collapsed
    },
    [uiMut.SET_DARK_MODE] (state, val: boolean) {
      state.darkMode = val
    },
    [uiMut.SET_DARK_MODE_UNLOCKED] (state, val: boolean) {
      state.darkModeUnlocked = val
    },
    [uiMut.SET_SHOW_JUMBOTRON] (state, val: boolean) {
      state.showJumbotron = val
    }
  }
}

export default ui
